@@ -10,6 +10,7 @@ import cartopy.crs as ccrs
 import pandas as pd
 import hvplot.pandas
 import hvplot.xarray
+import hvplot
 from bokeh.plotting import show
 import xarray as xr
 import warnings
@@ -129,29 +130,40 @@ def ctrc():
     print("Press Ctrl+C to stop plotting server")
 
 
-def in_notebook():
+def in_notebook(out = None):
     """
     Returns ``True`` if the module is running in IPython kernel,
     ``False`` if in IPython shell or other Python shell.
     """
+
+    if out is not None:
+        return True
+
     if "spyder" in sys.modules:
         return False
 
     return "ipykernel" in sys.modules
 
 
-def view(x, vars=None, autoscale=True):
+def view(x, vars=None, autoscale=True, out = None):
     """
-    Plot the contents of a NetCDF file
+    Plot the contents of a NetCDF out
     Parameters
     -------------
     x : object or str
-        xarray object or file path
+        xarray object or file  path
     vars : list or str
         Variables you want to plot. Everything will be plotted if this is not supplied
 
+    out : str
+        Name of file if you want to store output
+
     """
 
+    if out is not None:
+        if out.endswith(".html") is False:
+            raise ValueError("out name must end with html")
+
     coastline = True
     if type(x) is xr.core.dataarray.DataArray:
         x = x.to_dataset()
@@ -294,8 +306,9 @@ def view(x, vars=None, autoscale=True):
         if len(ds[lat_name].values) > 1:
             n_points += len(ds[lat_name].values)
 
-    if "lon" not in lon_name:
-        coastline = False
+    if lon_name is not None:
+        if "lon" not in lon_name:
+            coastline = False
 
     ff_dims = list(ds.coords)
 
@@ -304,7 +317,10 @@ def view(x, vars=None, autoscale=True):
     if len(possible_others) == 0:
         n_levels = 1
     else:
-        n_levels = len(ds[possible_others[0]].values)
+        if ds[possible_others[0]].values.ndim == 0:
+            n_levels = 0
+        else:
+            n_levels = len(ds[possible_others[0]].values)
 
     if vars is None:
         vars = [x for x in list(ds.variables) if x not in ff_dims]
@@ -417,10 +433,14 @@ def view(x, vars=None, autoscale=True):
         intplot = df.hvplot(
             groupby="variable",
             dynamic=True,
-            responsive=(in_notebook() is False),
+            responsive=(in_notebook(out) is False),
         )
-        if in_notebook():
-            return intplot
+        if in_notebook(out):
+            if out is None:
+                return intplot
+            else:
+                hvplot.save(intplot, out)
+                return None
 
         t = Thread(target=ctrc)
         t.start()
@@ -521,7 +541,7 @@ def view(x, vars=None, autoscale=True):
                             y_var,
                             vars,
                             cmap="RdBu_r",
-                            responsive=(in_notebook() is False),
+                            responsive=(in_notebook(out) is False),
                         ).redim.range(**{vars: (-v_max, v_max)})
                     else:
                         intplot = ds.hvplot.image(
@@ -530,7 +550,7 @@ def view(x, vars=None, autoscale=True):
                             vars,
                             cmap="RdBu_r",
                             rasterize=False,
-                            responsive=(in_notebook() is False),
+                            responsive=(in_notebook(out) is False),
                         ).redim.range(**{vars: (-v_max, v_max)})
 
                 else:
@@ -539,8 +559,12 @@ def view(x, vars=None, autoscale=True):
                     else:
                         intplot = ds.hvplot.image(x_var, y_var, vars, cmap="viridis")
 
-            if in_notebook():
-                return intplot
+            if in_notebook(out):
+                if out is None:
+                    return intplot
+                else:
+                    hvplot.save(intplot, out)
+                    return None
 
             t = Thread(target=ctrc)
             t.start()
@@ -646,8 +670,12 @@ def view(x, vars=None, autoscale=True):
                             x_var, y_var, vars, cmap="viridis", rasterize=False
                         )
 
-                    if in_notebook():
-                        return intplot
+                    if in_notebook(out):
+                        if out is None:
+                            return intplot
+                        else:
+                            hvplot.save(intplot, out)
+                            return None
 
                     t = Thread(target=ctrc)
                     t.start()
@@ -688,11 +716,15 @@ def view(x, vars=None, autoscale=True):
             .hvplot(
                 groupby="variable",
                 dynamic=True,
-                responsive=(in_notebook() is False),
+                responsive=(in_notebook(out) is False),
             )
         )
-        if in_notebook():
-            return intplot
+        if in_notebook(out):
+            if out is None:
+                return intplot
+            else:
+                hvplot.save(intplot, out)
+                return None
 
         t = Thread(target=ctrc)
         t.start()
@@ -721,7 +753,7 @@ def view(x, vars=None, autoscale=True):
                 coastline=coastline,
                 projection=projection,
                 rasterize=rasterize,
-                responsive=in_notebook() is False,
+                responsive=in_notebook(out) is False,
             )
         else:
 
@@ -740,11 +772,15 @@ def view(x, vars=None, autoscale=True):
                 coastline=coastline,
                 rasterize=False,
                 projection=projection,
-                responsive=in_notebook() is False,
+                responsive=in_notebook(out) is False,
             )
 
-        if in_notebook():
-            return intplot
+        if in_notebook(out):
+            if out is None:
+                return intplot
+            else:
+                hvplot.save(intplot, out)
+                return None
 
         t = Thread(target=ctrc)
         t.start()
@@ -785,7 +821,7 @@ def view(x, vars=None, autoscale=True):
                         coastline=coastline,
                         projection=projection,
                         rasterize=rasterize,
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     ).redim.range(**{vars: (-v_max, v_max)})
                 else:
                     intplot = ds.hvplot.quadmesh(
@@ -797,7 +833,7 @@ def view(x, vars=None, autoscale=True):
                         coastline=coastline,
                         projection=projection,
                         rasterize=rasterize,
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     )
                 # intplot = pn.Row(pn.WidgetBox(w1), intplot)
             else:
@@ -817,7 +853,7 @@ def view(x, vars=None, autoscale=True):
                         projection=projection,
                         rasterize=False,
                         cmap="RdBu_r",
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     ).redim.range(**{vars: (-v_max, v_max)})
                 else:
                     intplot = ds.hvplot.image(
@@ -829,11 +865,15 @@ def view(x, vars=None, autoscale=True):
                         projection=projection,
                         rasterize=False,
                         cmap="RdBu_r",
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     )
 
-            if in_notebook():
-                return intplot
+            if in_notebook(out):
+                if out is None:
+                    return intplot
+                else:
+                    hvplot.save(intplot, out)
+                    return None
 
             t = Thread(target=ctrc)
             t.start()
@@ -859,7 +899,7 @@ def view(x, vars=None, autoscale=True):
                         coastline=coastline,
                         projection=projection,
                         rasterize=rasterize,
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     ).redim.range(**{vars: (self_min.values, v_max)})
                 else:
                     intplot = ds.hvplot.quadmesh(
@@ -871,7 +911,7 @@ def view(x, vars=None, autoscale=True):
                         coastline=coastline,
                         projection=projection,
                         rasterize=rasterize,
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     )
 
             else:
@@ -892,7 +932,7 @@ def view(x, vars=None, autoscale=True):
                         coastline=coastline,
                         rasterize=False,
                         projection=projection,
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     ).redim.range(**{vars: (self_min.values, v_max)})
                 else:
                     intplot = ds.hvplot.image(
@@ -904,11 +944,15 @@ def view(x, vars=None, autoscale=True):
                         coastline=coastline,
                         rasterize=False,
                         projection=projection,
-                        responsive=(in_notebook() is False),
+                        responsive=(in_notebook(out) is False),
                     )
 
-            if in_notebook():
-                return intplot
+            if in_notebook(out):
+                if out is None:
+                    return intplot
+                else:
+                    hvplot.save(intplot, out)
+                    return None
 
             t = Thread(target=ctrc)
             t.start()
